#pragma once

#include <amdis/linearalgebra/Common.hpp>
#include <amdis/linearalgebra/DOFMatrixBase.hpp>

namespace AMDiS
{
  template <class Matrix>
  struct Constraints
  {
    template <class BitVec>
    static auto dirichletBC(Matrix& matrix, BitVec const& nodes, bool setDiagonal = true)
    {
      /* do nothing */
      return std::array<Triplet<typename Matrix::value_type>,0>{};
    }

    template <class BitVec, class Assoc>
    static auto periodicBC(Matrix& matrix, BitVec const& left, Assoc const& association, bool setDiagonal = true)
    {
      /* do nothing */
      return std::array<Triplet<typename Matrix::value_type>,0>{};
    }
  };

  template <class RB, class CB, class Backend>
  struct Constraints<DOFMatrixBase<RB,CB,Backend>>
  {
    using Matrix = DOFMatrixBase<RB,CB,Backend>;

    template <class BitVec>
    static auto dirichletBC(Matrix& matrix, BitVec const& nodes, bool setDiagonal = true)
    {
      return Constraints<typename Matrix::BaseMatrix>::dirichletBC(matrix.matrix(), nodes, setDiagonal);
    }

    template <class BitVec, class Assoc>
    static auto periodicBC(Matrix& matrix, BitVec const& left, Assoc const& association, bool setDiagonal = true)
    {
      return Constraints<typename Matrix::BaseMatrix>::periodicBC(matrix.matrix(), left, association, setDiagonal);
    }
  };

} // end namespace AMDiS
